#######################################################################################
#Code to run Spatial Occupancy Model for LICH with elevation as a covariate on colonization,
#drought index (PDSI) as a covariate on extinction (now time-varying), and stochasticity
#Written by Paige Howell, Richard Chandler
#Updated 09/01/2019 
#######################################################################################

# load libraries and data 
library(raster)
library(gdistance)
library(coda)
library(parallel)

#load data
#site coordinates
coords1<-read.csv("coords.dryad.csv")
coords<-data.matrix(coords1[,2:3], 274)

#site hydroperiod
watercov1<-read.csv("water.dryad.csv")
watercov<-as.character(watercov1[,2])

#presence-absence data
y1<-read.csv("y.wide.dryad.csv")
y.pre1<-data.matrix(y1[,2:45], 47)
y.pre<-array(y.pre1, dim=c(47, 3, 15))

#occasion covariates - wind, temp
wind1<-read.csv("wind.wide.dryad.csv")
wind.pre1<-data.matrix(wind1[,2:45], 47)
wind.pre<-array(wind.pre1, dim=c(47, 3, 15))

temp1<-read.csv("temp.wide.dryad.csv")
temp.pre1<-data.matrix(temp1[,2:45], 47)
temp.pre<-array(temp.pre1, dim=c(47, 3, 15))

#elevation
elevation=raster("dem_p_e_90m") #90m x 90m res, clipped to study area
dem.900mag<-aggregate(elevation, fact=10)
dem.900mag.scale <- scale(dem.900mag) 

#pdsi
pdsi1<-read.csv("pdsi.csv")
pdsi<-pdsi1[,2]

#Take a look at the raw data

sum(y.pre, na.rm=TRUE)                            # Total dets
apply(y.pre, 3, sum, na.rm=TRUE)                  # Dets per year
colSums(apply(y.pre, c(1,3), sum, na.rm=TRUE)>0)  # Sites occupied

nSampled <- nrow(y.pre)

devAskNewPage(TRUE)
for(t in 1:dim(y.pre)[3]) {
  dets.t <- which(rowSums(y.pre[,,t], na.rm=TRUE)>0)
  plot(dem.900mag.scale, main=paste("Year", (2003:2017)[t], "with", length(dets.t), "sites occupied", asp=1))
  points(coords, pch=3)
  points(coords[1:nSampled,], pch=16)
  points(coords[dets.t,], pch=16, col=rgb(0,0,1,0.9))
}

#load mcmc sampler
source("lich-resist-hydroCol-modavg-mcmc.R")

## Run in parallel
library(parallel)

cl1 <- makeCluster(3) #makeCluster(3, fork=TRUE) # Create 3 parallel instances of R

clusterSetRNGStream(cl = cl1, 348) # Set random seed for each

clusterExport(cl=cl1, c("dynroccHC", "y.pre", "watercov", "coords", "dem.900mag.scale", "wind.pre", "temp.pre", "pdsi"))

## Here is an example where we are estimating the influence of landscape resistance on colonization (estAlpha=TRUE), using Gibbs Variable selection (GVS=TRUE), not keeping track of likelihood of y and z (calc.waic=FALSE), and using the Norm(0, 1.7) prior for parameters in logit-linear models (prior.sens = 2))

system.time({
  outgvs.p1 <- clusterEvalQ(cl1, {
    library(gdistance)
    library(compiler)
    out.p1 <- dynroccHC(y=y.pre, x=coords, 
                         r.cov1=dem.900mag.scale, 
                         e.cov=watercov, e.cov2=pdsi,
                         p.cov1=wind.pre, p.cov2=temp.pre,
                         nIter=3,
                         tune=c(0.38, 0.01, 0.15, 0.15, #sig + gamma
                                9.1, 10.1, 3.7, 3.4, #eps
                                0.82, 0.86, 0.8, #det.
                                0.42, # alpha 
                                0.5, 0.5, 0.5, # w1-3
                                0.14, 0.1), # env. stoch.
                         monitor.z=TRUE,
                         estAlpha=TRUE,
                         calc.waic = FALSE,
                         prior.sens = 2,
                         GVS = TRUE,
                         report=0, plot.z=FALSE, tol=1e-3)
    return(out.p1)
  })
}) 
mcgvs.p1<-mcmc.list(lapply(outgvs.p1, function(x) mcmc(x$samples)))
mcgvsmat.p1<-as.matrix(mcgvs.p1)
windows();plot(mcgvs.p1, ask=TRUE)

#This software has been approved for release by the U.S. Geological Survey (USGS). Although the software has been subjected to rigorous review, the software has not been approved as a software release, and is only intended to document methodology.  The USGS reserves the right to update the software as needed pursuant to further analysis and review. No warranty, expressed or implied, is made by the USGS or the U.S. Government as to the functionality of the software and related material nor shall the fact of release constitute any such warranty. Furthermore, the software is released on condition that neither the USGS nor the U.S. Government shall be held liable for any damages resulting from its authorized or unauthorized use.
