## Script for running PVA simulations forward in time 
## Written by Paige Howell and Richard Chandler
## Updated September 1, 2019

# Load data and model output 
library(coda)
load("outgvs.p1.gzip")

# Forecast function
forecast <- function(mcout,         # Output from fitting MCMC sampler to data (mcmc object)
                     restore=NULL,     # Number of ponds to restore
                     fail=FALSE,       # Should ponds be allowed to fail structurally 
                     drought=FALSE,    # Drought effect
                     resistance=FALSE, # Estimate landscape resistance
                     stoch=FALSE,      # Environmental stochasticity
                     nfuture,          # Number of years to forecast
                     predgrid=NULL,    # Output grid of forecasted colonization probability
                     thin=1,           # Thin chains
                     start=1,          # Start (burn-in)
                     NbarSims=0, ...) {

  library(coda)
  library(gdistance)

  post.pars.mc <- window(as.mcmc.list(lapply(mcout, function(x) mcmc(x$samples))),
                         thin=thin, start=start)
  post.pars <- as.matrix(post.pars.mc)
  niter0 <- nrow(mcout[[1]]$samples)
  niter <- nrow(post.pars)
  nsites <- nrow(mcout[[1]]$final.state$z)
  nyears <- ncol(mcout[[1]]$final.state$z)
  post.zK.mc <- window(as.mcmc.list(lapply(mcout, function(x) mcmc(t(x$zK)))),
                       thin=thin, start=start)
  post.zK <- as.matrix(post.zK.mc)
  nchains <- nchain(post.pars.mc)
  post.zA0 <- array(NA, c(nsites, nyears, niter0*nchains))
  for(i in 1:nchains) {
    post.zA0[,,(1:niter0)+niter0*(i-1)] <- mcout[[i]]$zA
  }
  post.zA <- post.zA0[,,seq(from=start,by=thin,length.out=niter)]

  z <- array(NA, c(nsites, nyears+nfuture, nrow(post.pars)))
  z[,1:nyears,] <- post.zA[,1:nyears,]

  #load data
  #site coordinates
  coords1<-read.csv("coords.dryad.csv")
  x<-data.matrix(coords1[,2:3], 274)
  
  #site hydroperiod
  watercov1<-read.csv("water.dryad.csv")
  watercov<-as.character(watercov1[,2])
  
  #elevation
  elevation=raster("dem_p_e_90m") #90m x 90m res, clipped to study area
  dem.900mag<-aggregate(elevation, fact=10)
  r.cov1 <- scale(dem.900mag) 
  
  #Sites no longer sampled because they were destroyed
  known0 <- rep(FALSE, nsites) 
  known0[c(10, 19, 31)] <- TRUE
  notFailed <- 1 - known0
  
  ## Known values (blowout=0 if failed, 1 otherwise)
  blowout <- matrix(1, nsites, (nyears+nfuture))
  blowout[c(10,19,31), 8:(nyears+nfuture)] <- 0
  
  isInter_d <-isInter_f <- watercov=="Intermittent"
  isSemi_d <- isSemi_f<-watercov=="Semi-permanent"
  isPerm_d <- isPerm_f<- watercov=="Permanent"
  
  water<-model.matrix(~ watercov-1)
  colnames(water)<-c("Inter", "Perm", "Semi")
  rownames(water)<-rownames(x)
  
  #set up dummy variables
  Inter_d <-Inter_f <-as.vector(water[,1])
  Perm_d <- Perm_f <-as.vector(water[,2])
  Semi_d <- Semi_f <-as.vector(water[,3])

  gamma0 <- rep(NA, nsites)
  gamma <- matrix(NA, nsites, nyears+nfuture)
  Egamma <- gamma
  
  epsilon<-matrix(NA, nsites, nyears+nfuture)

  Ez <- array(NA_real_, c(nsites, nyears+nfuture, nrow(post.pars)))

  do.grid <- !is.null(predgrid)
  if(do.grid) {
    ngridpts <- nrow(predgrid)
    gammaGrid <- array(NA, c(ngridpts, nyears+nfuture, nrow(post.pars)))
  }

  ##Which sites should be restored?
  ##Sites that have a high probability of colonization, not permanent
  ##Sites differ depending on whether we are considering pdsi or not, resistance or not

  #management actions
  if(!is.null(restore)) {
      coords.m<-matrix(x, 274, 2)
      rownames(coords.m)<-rownames(x)

      if(restore==1){ # 1 pond restored
        restoredi<-x[273,,drop=FALSE] 
      }
      if(restore==3){ # 3 ponds restored
        restoredi<-x[c(273, 141, 142),,drop=FALSE] 
      }
      if(restore==6){ # 6 ponds restored
        restoredi<-x[c(273, 141, 142, 143, 144, 39),,drop=FALSE]
      }
     }


  if(!is.null(restore)) { # change the hydroperiod of the sites being managed
    restored <- rownames(coords.m) %in% rownames(restoredi)
    Inter_f[restored]<-0
    Semi_f[restored]<-0
    Perm_f[restored]<-1

    isInter_f[restored]<-"FALSE"
    isInter_f<-as.logical(isInter_f)
    isSemi_f[restored]<-"FALSE"
    isSemi_f<-as.logical(isSemi_f)
    isPerm_f[restored]<-"TRUE"
    isPerm_f<-as.logical(isPerm_f)

  }

    do.Nbar <- NbarSims>0
    if(do.Nbar) {
        nSims <- NbarSims
    } else {
        nSims <- 1
    }
    N <- array(NA_integer_, c(nyears+nfuture, nSims, niter))

    pdsi1<-read.csv("pdsi.csv")
    pdsi<-as.vector(pdsi1[,2])
    pdsi.mean <- mean(pdsi)
    pdsi.sd <- sd(pdsi)
    pdsi.s <- (pdsi-pdsi.mean)/pdsi.sd
    pyears <- c(2003:2017)-2010 ## Center years, roughly
    pdsi.lm <- lm(pdsi~pyears)
    fyears <- seq(from=2018, length.out=nfuture)-2010
    E.pdsi <- cbind(1, fyears) %*% coef(pdsi.lm)
    ESD.pdsi <- sigma(pdsi.lm)

  for(s in 1:niter) {
    if(s %% 100 == 0) cat("  Doing sample", s, "of", niter, "\n")

    w1 <- post.pars[s,"w1"]
    cost <- exp(w1*post.pars[s,"alpha1"]*r.cov1)
    tr1 <- transition(cost, transitionFunction=function(x) 1/mean(x), directions=16)
    tr1CorrC <- geoCorrection(tr1, type="c", multpl=FALSE,scl=FALSE)
    D <- costDistance(tr1CorrC,x,x)/1000

    gamma0[isInter_d] <- post.pars[s,"gamma0.i"]
    gamma0[isSemi_d] <- post.pars[s,"gamma0.s"]
    gamma0[isPerm_d] <- post.pars[s,"gamma0.p"]

    epsilon.i <- post.pars[s,"epsilon.i"]
    epsilon.s <- post.pars[s,"epsilon.s"]
    epsilon.p <- post.pars[s,"epsilon.p"]

    sdEta <- post.pars[s,"sdEta"]

    w2 <- post.pars[s,"w2"]
    w3 <- post.pars[s,"w3"]
    epsilon.c <- post.pars[s, "epsilon.c"]

    G <- gamma0*exp(-D^2/(2*post.pars[s,"sigma"]^2))
    if(do.grid) {
      Dgrid <- costDistance(tr1CorrC,x,predgrid)/1000
      Ggrid <- gamma0*exp(-Dgrid^2/(2*post.pars[s,"sigma"]^2))
    }

    eta.d <- post.pars[s, paste("eta", 1:14, sep="")]
    for(k in 2:nyears) {

      epsilon[,k-1] <- plogis(epsilon.i*Inter_d + epsilon.s*Semi_d +
                              epsilon.p*Perm_d + w2*epsilon.c*pdsi.s[k-1] +
                              w3*eta.d[k-1])

      PrNotColonizedByNeighbor <- 1 - G * t(z[,rep(k-1, nsites),s])
      gamma[,k-1] <- 1 - exp(rowSums(log(PrNotColonizedByNeighbor)))
      Ez[,k-1,s] <- (z[,k-1,s]*(1-epsilon[,k-1]*(1-gamma[,k-1])) +
                       (1-z[,k-1,s])*gamma[,k-1])*blowout[,k-1]
      if(do.grid) {
        gammaGrid[,k-1,s] <- 1-exp(rowSums(log(1-t(Ggrid)*t(z[,rep(k-1, ngridpts),s]))))
      }
    }

    gamma0 <- rep(NA, nsites)
    gamma0[isInter_f] <- post.pars[s,"gamma0.i"]
    gamma0[isSemi_f] <- post.pars[s,"gamma0.s"]
    gamma0[isPerm_f] <- post.pars[s,"gamma0.p"]

    G <- gamma0*exp(-D^2/(2*post.pars[s,"sigma"]^2))

    if(do.grid) {
      Dgrid <- costDistance(tr1CorrC,x,predgrid)/1000
      Ggrid <- gamma0*exp(-Dgrid^2/(2*post.pars[s,"sigma"]^2))
    }

    ## The q loop is used to compute the expected value of N for a given posterior draw
    ## Only the last value (q=nSims) of z, Ez, etc... will be stored
    for(q in 1:nSims) {

    eta <- rnorm((nyears+nfuture-1), 0, sdEta)
    eta[1:14] <- eta.d

    N.d <- colSums(z[,1:15,s])

    for(k in (nyears+1):(nyears+nfuture)) {

      ## Probability of a tank failing and remaining blown out
      ## Number of blowouts (3) divided by sites sampled x years surveyd (47*11)
      if(fail){
          for(i in 1:nsites){
              ## blowout=1 if the tank is viable. blowout=0 if the tank failed
          blowout[i,k] <- rbinom(1, 1, (1-0.005802708)*blowout[i,k-1])
        }
      }

      if(drought) {
          ## Need to average over the prior for the betas
          library(mvtnorm)
          pdsi.beta <- t(rmvnorm(1, coef(pdsi.lm), vcov(pdsi.lm)))
          E.pdsi <- cbind(1, fyears) %*% pdsi.beta 
          pdsi.f <- rnorm(nfuture, E.pdsi, ESD.pdsi)
          pdsi.fs <- (pdsi.f-pdsi.mean)/pdsi.sd
      } else {
          pdsi.f <- rnorm(nfuture, mean(pdsi), sd(pdsi)) 
          pdsi.fs <- (pdsi.f-pdsi.mean)/pdsi.sd
      }
      pdsit<-c(pdsi, pdsi.fs)
      epsilon[,k-1] <- plogis(epsilon.i*Inter_f + epsilon.s*Semi_f +
                              epsilon.p*Perm_f + w2*epsilon.c*pdsit[k-1] +
                              w3*eta[k-1])

      PrNotColonizedByNeighbor <- 1 - G * t(z[,rep(k-1, nsites),s])
      gamma[,k-1] <- 1 - exp(rowSums(log(PrNotColonizedByNeighbor)))
      Ez[,k-1,s] <- (z[,k-1,s]*(1-epsilon[,k-1]*(1-gamma[,k-1])) +
                     (1-z[,k-1,s])*gamma[,k-1])*blowout[,k-1]
      z[,k,s] <- rbinom(nsites, 1, Ez[,k-1,s])
      if(do.grid) {
        gammaGrid[,k-1,s] <- 1-exp(rowSums(log(1-t(Ggrid)*t(z[,rep(k-1, ngridpts),s]))))
      }
      N[1:15,q,s] <- N.d
      N[k,q,s] <- sum(z[,k,s])
      }
    }
  }

  out <- list(z=z, blowout=blowout, Ez=Ez, 
              N=N,
              epsilon=epsilon, gamma=gamma)
  if(do.grid)
    out$gammaGrid <- gammaGrid

  return(out)
}

# Run projections (new) 
#debugonce(forecast)

fortest <- forecast(outgvs.p1, restore=NULL, fail=TRUE, drought=TRUE, stoch=TRUE,
                    resistance=TRUE, nfuture=2, thin=1, start=1,
                    NbarSims=2)

str(fortest)

#This software has been approved for release by the U.S. Geological Survey (USGS). Although the software has been subjected to rigorous review, the software has not been approved as a software release, and is only intended to document methodology.  The USGS reserves the right to update the software as needed pursuant to further analysis and review. No warranty, expressed or implied, is made by the USGS or the U.S. Government as to the functionality of the software and related material nor shall the fact of release constitute any such warranty. Furthermore, the software is released on condition that neither the USGS nor the U.S. Government shall be held liable for any damages resulting from its authorized or unauthorized use.