library(compiler)

## MCMC algorithm
dynroccH <- function(y,            # nSampled x nVisits x nYear array of detection/non-detection data
                    x,             # nSites x 2 matrix of site coordinates. Note that nSampled will usually be <nSites
                    r.cov1,        # resistance covariate
                    r.cov2=NULL,   # resistance covariate
                    e.cov,         # extinction covariate
                    e.cov2,        # extinction covariate
                    p.cov1,        # detection covariate
                    p.cov2,        # detection covariate
                    prior.sens = 1, # To check for prior sensitivity, vary this between 1 and 3 to run algorithm with three different prior distributions for parameters in logit-linear models
                    nIter=10,      # MCMC iterations
                    tune,          # Tuning order: sigma,gamma0.i,gamma0.s,gamma0.p,eps.i,eps.s,eps.p,                                                      eps.c,beta0,beta1,beta2,alpha[1],alpha[2],w1,w2,w3,
                                   #               sdEta, eta
                    estAlpha=TRUE, # Estimate the resistance coefficient?
                    inits=NULL,    # until you run algorithm, inits are based on what is given.
                    zProp=c("ind","vec"), # Update z matrix by either proposing z(i,k) or z(,k), respectively
                    zProbs=NULL,     # matrix of proposal probs use if zProp="vec"
                    monitor.z=FALSE, # store each iteration of the z matrix?
                    calc.waic=FALSE, # store z and y likelihood?
                    GVS=FALSE,       # Do Gibbs Variable Selection?
                    report=0,        # Only report progress if >0
                    plot.z=FALSE,    # Plot the latent occupancy state (if report>0)
                    tol=0)           # This will reject a proposal of z(i,k)=1 if mu(i,k-1)<tol
{

  zProp <- match.arg(zProp)

  ## Dimensions
  nSites <- nrow(x) 
  nReps <- ncol(y)
  nYears <- dim(y)[3]
  
  ## Use this to avoid likelihood calculations for sites not sampled
  nSampled <- nrow(y)
  dataYears <- apply(!is.na(y), 3, any)

  anyDetections <- matrix(FALSE, nSites, nYears)
  anyDetections[1:nSampled,] <- apply(y, c(1,3), sum, na.rm=TRUE) > 0

  #Sites no longer sampled because they were destroyed
  known0 <- matrix(FALSE, nSites, nYears)
  known0[c(10, 19, 31), 8:nYears]<-TRUE
  notFailed <- 1 - known0

  if(any(anyDetections & known0))
      stop("detection data doesn't match blowout data")

  y.wide <- matrix(y, nSampled)

  isInter <- e.cov=="Intermittent"
  isSemi <- e.cov=="Semi-permanent"
  isPerm <- e.cov=="Permanent"
  
  water<-model.matrix(~ e.cov-1)
  colnames(water)<-c("Inter", "Perm", "Semi")
  rownames(water)<-rownames(x)
  
  Inter<-water[,1] 
  Perm<-water[,2]
  Semi<-water[,3]
  
  pdsi<-e.cov2
  eta<-rep(NA, nYears-1) 
  gamma0 <- rep(NA, nSites)  

  rc2 <- is.null(r.cov2)
  
  if(rc2) {r.cov2 <- r.cov1}

  ## initial values
    gamma <- muz <- epsilon<- matrix(NA, nSites, nYears-1)
    if(is.null(inits)) {
      w1<-w2<-w3<-1 # For Gibbs variable selection
      epsilon.p<-epsilon.s<-epsilon.i<-epsilon.c<-runif(1) 
      sdEta <- rgamma(1, 1, 1)
      eta <- rnorm(nYears-1, 0, 0.01)
      sigma <- runif(1,3,4)
      gamma0.i <- runif(1, 0.01, 0.3)
      gamma0.s <- runif(1, 0.01, 0.3)
      gamma0.p <- runif(1, 0.01, 0.3)
      gamma0[isInter] <- gamma0.i
      gamma0[isSemi] <- gamma0.s
      gamma0[isPerm] <- gamma0.p
      beta0 <- rnorm(1)
      beta1 <- rnorm(1)
      beta2 <- rnorm(1)
      alpha1 <- 0 
      p <- plogis(beta0 + beta1*p.cov1 + beta2*p.cov2)
      z <- matrix(0, nSites, nYears)
      z[c(15, 33, 274),1] <- 1  # reintroduction sites
      cost <- exp(w1*alpha1*r.cov1) # create resistance surface
      ## calculate conductances among neighbors
      tr1 <- transition(cost, transitionFunction=function(x) 1/mean(x), directions=16) 
      tr1CorrC <- geoCorrection(tr1, type="c", multpl=FALSE,scl=FALSE) #adjust diagonal conductances
      ## calculate least cost distance between all pairs of sites.
      if(!estAlpha)
          alpha1 <- 0 ## Force alpha to be 0 if you aren't estimating it. Results in appox Euclidean dist
      D <- costDistance(tr1CorrC,x,x)/1000 # calculate the ecological distance matrix
      G <- gamma0*exp(-D^2/(2*sigma^2))

      for(k in 2:nYears) { 
          epsilon[,k-1]<-plogis(epsilon.i*Inter + epsilon.s*Semi + epsilon.p*Perm + w2*epsilon.c*e.cov2[k-1] + w3*eta[k-1])
          PrNotColonizedByNeighbor <- 1 - gamma0*exp(-D^2/(2*sigma^2)) * t(z[,rep(k-1, nSites)])
          PrNotColonizedAtAll <- apply(PrNotColonizedByNeighbor, 1, prod)
          gamma[,k-1] <- 1 - PrNotColonizedAtAll
          muz[,k-1] <- z[,k-1]*(1-epsilon[,k-1]*(1-gamma[,k-1])) + (1-z[,k-1])*gamma[,k-1] 
          muz[,k-1] <- muz[,k-1]*notFailed[,k] 
          z[,k] <- rbinom(nSites, 1, muz[,1])
          z[known0[,k],k] <- 0 
          z[which(anyDetections[,k]),k] <- 1 
      }
  } else {
    
    if(GVS){
      w1 <- inits$samples["w1"]
      w2 <- inits$samples["w2"]
      w3 <- inits$samples["w3"]
      }
      gamma0.i <- inits$samples["gamma0.i"]
      gamma0.s <- inits$samples["gamma0.s"]
      gamma0.p <- inits$samples["gamma0.p"]
      gamma0[isInter] <- gamma0.i
      gamma0[isSemi] <- gamma0.s
      gamma0[isPerm] <- gamma0.p
      sigma <- inits$samples["sigma"]
      epsilon.i <- inits$samples["epsilon.i"]
      epsilon.s <- inits$samples["epsilon.s"]
      epsilon.p <- inits$samples["epsilon.p"]
      epsilon.c <- inits$samples["epsilon.c"]
      sdEta<-inits$samples["sdEta"]
      eta<-inits$eta
      alpha1<-inits$samples["alpha1"]
      D <- inits$D
      beta0 <- inits$samples["beta0"]
      beta1 <- inits$samples["beta1"]
      beta2 <- inits$samples["beta2"]
      p <- plogis(beta0 + beta1*p.cov1 + beta2*p.cov2)
      z <- inits$z
      .Random.seed <- inits$seed 
  }

    ##  browser() - troubleshoot

    # For WAIC, track the likelihood for each observation (latent and data)
    ll.z <- matrix(0, nSites, nYears)
    ll.y <- array(0, c(nSampled, nReps, nYears))
    
  for(k in 2:nYears) {
      epsilon[,k-1]<-plogis(epsilon.i*Inter + epsilon.s*Semi + epsilon.p*Perm + w2*epsilon.c*e.cov2[k-1] + w3*eta[k-1])
      PrNotColonizedByNeighbor <- 1 - gamma0*exp(-D^2/(2*sigma^2  ))*t(z[,rep(k-1, nSites)])
      PrNotColonizedAtAll <- apply(PrNotColonizedByNeighbor, 1, prod)
      gamma[,k-1] <- 1 - PrNotColonizedAtAll
      muz[,k-1] <- z[,k-1]*(1-epsilon[,k-1]*(1-gamma[,k-1])) + (1-z[,k-1])*gamma[,k-1] 
      muz[,k-1] <- muz[,k-1]*notFailed[,k]
      ll.z[,k-1] <- dbinom(z[,k], 1, muz[,k-1], log=TRUE)
      if(k > 4) { ## Ignore first 4 years without data
        ll.y[,,k] <- dbinom(y[,,k], 1, z[1:nSampled,k]*p[,,k], log=TRUE)
      }
  }
  eta.cand <- eta
  ll.z.cand <- ll.z
  ll.z.sum <- sum(ll.z)
  ll.y.cand <- ll.y
  ll.y.sum <- sum(ll.y, na.rm=TRUE)
  epsilon.cand <- epsilon
  gamma.cand <- gamma
  muz.cand <- muz

  nz1 <- z ## Used to compute expected occupancy at each site
  zkup <- rep(0, nYears-1)

  ## posterior samples
  nPar <- 18+nYears+(nYears-1)
  samples <- array(NA, c(nIter, nPar))
  zK <- matrix(NA, nSites, nIter)
  colnames(samples) <- c("w1", "w2", "w3","sigma", "gamma0.i", "gamma0.s", "gamma0.p",
                         "epsilon.i", "epsilon.s","epsilon.p", "epsilon.c", "sdEta",
                         "beta0", "beta1", "beta2", "alpha1", 
                         paste("eta", 1:(nYears-1), sep=""),
                         paste("zk", 1:nYears, sep=""), "deviance", "deviance.latent")
  reportit <- report>0
  nzup <- rep(0, nYears-1)
  zA <- NULL
  lya <- NULL
  lza <- NULL
  
  if(calc.waic)
    lya <- array(NA_integer_, c(nSampled, nReps, nYears, nIter))
    lza <- array(NA_integer_, c(nSites, nYears, nIter))
  
  if(monitor.z)
      zA <- array(NA_integer_, c(nSites, nYears, nIter))

  if(reportit) {
      cat("iter 1\n")
      cat("    theta =", round(c(w1, w2, w3, sigma, gamma0.i,gamma0.s,gamma0.p,epsilon.i,epsilon.s,epsilon.p,epsilon.c, sdEta,beta0,beta1,beta2,alpha1), 5), "\n")
      cat("    eta =", round(eta, 5), "\n")
      cat("    z[k] =", round(colSums(z), 2), "\n")
      cat("    ll.z =", round(sum(ll.z), 2), "\n")
      cat("    deviance =", round(-2*ll.y.sum, 2), "\n")
      cat("    deviance.latent=", round(-2*ll.z.sum, 2), "\n")
      cat("    time =", format(Sys.time()), "\n")
      if(plot.z) {
          library(lattice)
          zd <- data.frame(z=as.integer(z), year=factor(rep(2003:2017, each=nSites)),
                           x=as.numeric(x[,1])/1000, y=as.numeric(x[,2])/1000)
          print(xyplot(y ~ x | year, zd, groups=z, aspect="iso", pch=c(1,16), as.table=TRUE))
      }
  }


  ## Sample from posteriors
  for(s in 1:nIter) {
    ll.z.sum <- sum(ll.z)

    if(reportit) {
    if(s %in% c(2:100) & s %% report == 0) {
      cat("iter", s, "\n")
      cat("    theta =", round(samples[s-1,1:16], 5), "\n")
      cat("    eta =", round(eta, 5), "\n")
      cat("    z[k] =", colSums(z), "\n")
      cat("    accepted", round(zkup/(nSites)*100, 1), "percent of z[k] proposals \n")
      cat("    sum(ll.z) =", ll.z.sum, "\n")
      cat("    deviance =", round(-2*ll.y.sum, 2), "\n")
      cat("    latent deviance =", round(samples[s-1,"deviance.latent"], 2), "\n")
      cat("    time =", format(Sys.time()), "\n")
      if(plot.z) {
          library(lattice)
          zd$z <- as.integer(z)
          print(xyplot(y ~ x | year, zd, groups=z, aspect="iso", pch=c(1,16), as.table=TRUE))
      }
    }
    }

    if(GVS){
    if(estAlpha) {

    #Metropolis update for w1
    w1.cand <- 1-w1 
    cost <- exp(w1.cand*alpha1*r.cov1) 
    tr1 <- transition(cost, transitionFunction=function(x) 1/mean(x), directions=16) 
    tr1CorrC <- geoCorrection(tr1, type="c", multpl=FALSE,scl=FALSE) 
    D.cand <- costDistance(tr1CorrC,x,x)/1000 
    G.cand <- gamma0*exp(-D.cand^2/(2*sigma^2))
    
    for(k in 2:nYears) {
      zkt <- matrix(z[,k-1], nSites, nSites, byrow=TRUE)
      gamma.cand[,k-1] <- 1 - exp(rowSums(log(1-G.cand*zkt)))
      muz.cand[,k-1] <- (z[,k-1]*(1-epsilon[,k-1]*(1-gamma.cand[,k-1])) + (1-z[,k-1])*gamma.cand[,k-1])*notFailed[,k]
      ll.z.cand[,k-1] <- dbinom(z[,k], 1, muz.cand[,k-1], log=TRUE)
    }
    
    ll.z.sum.cand <- sum(ll.z.cand)
    
      prior.w1.cand <- dbinom(w1.cand, 1, 0.5, log=TRUE)
      prior.w1 <- dbinom(w1, 1, 0.5, log=TRUE)
    
    if(runif(1) < exp((ll.z.sum.cand + prior.w1.cand) -
                      (ll.z.sum + prior.w1))) {
      w1 <- w1.cand
      D <- D.cand
      G <- G.cand
      gamma <- gamma.cand
      muz <- muz.cand
      ll.z <- ll.z.cand
      ll.z.sum <- ll.z.sum.cand
    }
    }
    
    ## Metropolis update for w2
    w2.cand <- 1-w2 
    
    for(k in 2:nYears) {
      epsilon.cand[,k-1]<-plogis(epsilon.i*Inter + epsilon.s*Semi + epsilon.p*Perm + w2.cand*epsilon.c*pdsi[k-1] + w3*eta[k-1])
      muz.cand[,k-1] <- z[,k-1]*(1-epsilon.cand[,k-1]*(1-gamma[,k-1])) + (1-z[,k-1])*gamma[,k-1] 
      muz.cand[,k-1] <- muz.cand[,k-1]*notFailed[,k]
      ll.z.cand[,k-1] <- dbinom(z[,k], 1, muz.cand[,k-1], log=TRUE)
    }
    ll.z.sum.cand <- sum(ll.z.cand)
    
    prior.w2.cand <- dbinom(w2.cand, size=1, prob=0.5, log=TRUE)
    prior.w2 <- dbinom(w2, size=1, prob=0.5, log=TRUE)
    
    if(runif(1) < exp((ll.z.sum.cand + prior.w2.cand) -
                      (ll.z.sum + prior.w2))) {
      
      w2<-w2.cand
      epsilon<- epsilon.cand
      muz <- muz.cand
      ll.z<-ll.z.cand
      ll.z.sum <- ll.z.sum.cand
    }
    
    ## Metropolis update for w3
    w3.cand <- 1-w3 
    for(k in 2:nYears) {
      epsilon.cand[,k-1]<-plogis(epsilon.i*Inter + epsilon.s*Semi + epsilon.p*Perm + w2*epsilon.c*pdsi[k-1] + w3.cand*eta[k-1])
      muz.cand[,k-1] <- z[,k-1]*(1-epsilon.cand[,k-1]*(1-gamma[,k-1])) + (1-z[,k-1])*gamma[,k-1]
      muz.cand[,k-1] <- muz.cand[,k-1]*notFailed[,k]
      ll.z.cand[,k-1] <- dbinom(z[,k], 1, muz.cand[,k-1], log=TRUE)
    }
      ll.z.sum.cand <- sum(ll.z.cand)

      prior.w3.cand <- dbinom(w3.cand, size=1, prob=0.5, log=TRUE)
      prior.w3 <- dbinom(w3, size=1, prob=0.5, log=TRUE)
      

    if(runif(1) < exp((ll.z.sum.cand + prior.w3.cand) -
                      (ll.z.sum + prior.w3))) {

      w3<-w3.cand      
      epsilon <- epsilon.cand
      muz <- muz.cand
      ll.z<-ll.z.cand
      ll.z.sum <- ll.z.sum.cand
    }
    }
      
    if(estAlpha) {
    #Metropolis update for alpha
    alpha1.cand <- rnorm(1, alpha1, tune[12])
    cost <- exp(w1*alpha1.cand*r.cov1) 
    tr1 <- transition(cost, transitionFunction=function(x) 1/mean(x), directions=16)
    tr1CorrC <- geoCorrection(tr1, type="c", multpl=FALSE,scl=FALSE) 
    D.cand <- costDistance(tr1CorrC,x,x)/1000
    G.cand <- gamma0*exp(-D.cand^2/(2*sigma^2))

    for(k in 2:nYears) {
      zkt <- matrix(z[,k-1], nSites, nSites, byrow=TRUE)
      gamma.cand[,k-1] <- 1 - exp(rowSums(log(1-G.cand*zkt)))
      muz.cand[,k-1] <- (z[,k-1]*(1-epsilon[,k-1]*(1-gamma.cand[,k-1])) + (1-z[,k-1])*gamma.cand[,k-1])*notFailed[,k]
      ll.z.cand[,k-1] <- dbinom(z[,k], 1, muz.cand[,k-1], log=TRUE)
      }

      if(w1==1){
        prior.alpha1.cand <- dnorm(alpha1.cand, 0, 10, log=TRUE)
        prior.alpha1 <- dnorm(alpha1, 0, 10, log=TRUE)
      }
    
      if(w1==0){
        prior.alpha1.cand <- dnorm(alpha1.cand, 0.9, 0.38, log=TRUE)
        prior.alpha1 <- dnorm(alpha1, 0.9, 0.38, log=TRUE)
      }
      
      ll.z.sum.cand <- sum(ll.z.cand)
      if(runif(1) < exp((ll.z.sum.cand + prior.alpha1.cand) -
                        (ll.z.sum + prior.alpha1))) {
      alpha1 <- alpha1.cand
      D <- D.cand
      G <- G.cand
      gamma <- gamma.cand
      muz <- muz.cand
      ll.z <- ll.z.cand
      ll.z.sum <- ll.z.sum.cand
      }
    }
  

    ## Metropolis update for sigma
    sigma.cand <- rnorm(1, sigma, tune[1])
    if(sigma.cand > 0) {
        G.cand <- gamma0*exp(-D^2/(2*sigma.cand^2))
      for(k in 2:nYears) {
        zkt <- matrix(z[,k-1], nSites, nSites, byrow=TRUE)
        gamma.cand[,k-1] <- 1 - exp(rowSums(log(1-G.cand*zkt)))
        muz.cand[,k-1] <- (z[,k-1]*(1-epsilon[,k-1]*(1-gamma.cand[,k-1])) + (1-z[,k-1])*gamma.cand[,k-1])*notFailed[,k]
        ll.z.cand[,k-1] <- dbinom(z[,k], 1, muz.cand[,k-1], log=TRUE)
      }
      prior.sigma.cand <- dgamma(sigma.cand, 0.001, 0.001)
      prior.sigma <- dgamma(sigma, 0.001, 0.001)
      ll.z.sum.cand <- sum(ll.z.cand)
      if(runif(1) < exp((ll.z.sum.cand + prior.sigma.cand) -
                        (ll.z.sum + prior.sigma))) {
          sigma <- sigma.cand
          gamma <- gamma.cand
          ll.z <- ll.z.cand
          ll.z.sum <- ll.z.sum.cand
          muz <- muz.cand
          G <- G.cand
      }
    }

    # Metropolis update for gamma0 (part of the gammaDist calculation)
    prior.gamma0.cand <- prior.gamma0 <- 0
    gamma0.i.cand <- rnorm(1, gamma0.i, tune[2])
    if(gamma0.i.cand > 0 & gamma0.i.cand < 1) {
        gamma0.cand <- gamma0
        gamma0.cand[isInter] <- gamma0.i.cand
        G.cand <- gamma0.cand*exp(-D^2/(2*sigma^2  ))
      for(k in 2:nYears) { 
        zkt <- matrix(z[,k-1], nSites, nSites, byrow=TRUE)
        gamma.cand[,k-1] <- 1 - exp(rowSums(log(1-G.cand*zkt)))
        muz.cand[,k-1] <-(z[,k-1]*(1-epsilon[,k-1]*(1-gamma.cand[,k-1])) + (1-z[,k-1])*gamma.cand[,k-1])*notFailed[,k]
        ll.z.cand[,k-1] <- dbinom(z[,k], 1, muz.cand[,k-1], log=TRUE)
    }
      ll.z.sum.cand <- sum(ll.z.cand)
      if(runif(1) < exp((ll.z.sum.cand + prior.gamma0.cand) -
                        (ll.z.sum + prior.gamma0))) {
        gamma0.i <- gamma0.i.cand
        gamma0 <- gamma0.cand
        gamma <- gamma.cand
        muz <- muz.cand
        ll.z <- ll.z.cand
        ll.z.sum <- ll.z.sum.cand
        G <- G.cand
    }
  }


    # Metropolis update for gamma0 (part of the gammaDist calculation)
    gamma0.s.cand <- rnorm(1, gamma0.s, tune[3])
    if(gamma0.s.cand > 0 & gamma0.s.cand < 1) {
        gamma0.cand <- gamma0
        gamma0.cand[isSemi] <- gamma0.s.cand
        G.cand <- gamma0.cand*exp(-D^2/(2*sigma^2  ))
      for(k in 2:nYears) { #nYears
        zkt <- matrix(z[,k-1], nSites, nSites, byrow=TRUE)
        gamma.cand[,k-1] <- 1 - exp(rowSums(log(1-G.cand*zkt)))
        muz.cand[,k-1] <-(z[,k-1]*(1-epsilon[,k-1]*(1-gamma.cand[,k-1])) + (1-z[,k-1])*gamma.cand[,k-1])*notFailed[,k]
        ll.z.cand[,k-1] <- dbinom(z[,k], 1, muz.cand[,k-1], log=TRUE)
    }
      ll.z.sum.cand <- sum(ll.z.cand)
      if(runif(1) < exp((ll.z.sum.cand + prior.gamma0.cand) -
                        (ll.z.sum + prior.gamma0))) {
        gamma0.s <- gamma0.s.cand
        gamma0 <- gamma0.cand
        gamma <- gamma.cand
        muz <- muz.cand
        ll.z <- ll.z.cand
        ll.z.sum <- ll.z.sum.cand
        G <- G.cand
    }
  }

    # Metropolis update for gamma0 (part of the gammaDist calculation)
    gamma0.p.cand <- rnorm(1, gamma0.p, tune[4])
    if(gamma0.p.cand > 0 & gamma0.p.cand < 1) {
        gamma0.cand <- gamma0
        gamma0.cand[isPerm] <- gamma0.p.cand
        G.cand <- gamma0.cand*exp(-D^2/(2*sigma^2  ))
      for(k in 2:nYears) { #nYears
        zkt <- matrix(z[,k-1], nSites, nSites, byrow=TRUE)
        gamma.cand[,k-1] <- 1 - exp(rowSums(log(1-G.cand*zkt)))
        muz.cand[,k-1] <-(z[,k-1]*(1-epsilon[,k-1]*(1-gamma.cand[,k-1])) + (1-z[,k-1])*gamma.cand[,k-1])*notFailed[,k]
        ll.z.cand[,k-1] <- dbinom(z[,k], 1, muz.cand[,k-1], log=TRUE)
    }
      ll.z.sum.cand <- sum(ll.z.cand)
      if(runif(1) < exp((ll.z.sum.cand + prior.gamma0.cand) -
                        (ll.z.sum + prior.gamma0))) {
        gamma0.p <- gamma0.p.cand
        gamma0 <- gamma0.cand
        gamma <- gamma.cand
        muz <- muz.cand
        ll.z <- ll.z.cand
        ll.z.sum <- ll.z.sum.cand
        G <- G.cand
    }
  }

    ## Metropolis update for epsilon (intermittent sites)
    epsilon.i.cand <- rnorm(1, epsilon.i, tune[5])

    for(k in 2:nYears) {
      epsilon.cand[,k-1]<-plogis(epsilon.i.cand*Inter + epsilon.s*Semi + epsilon.p*Perm + w2*epsilon.c*pdsi[k-1] + w3*eta[k-1])
      muz.cand[,k-1] <- z[,k-1]*(1-epsilon.cand[,k-1]*(1-gamma[,k-1])) + (1-z[,k-1])*gamma[,k-1] 
      muz.cand[,k-1] <- muz.cand[,k-1]*notFailed[,k]
      ll.z.cand[,k-1] <- dbinom(z[,k], 1, muz.cand[,k-1], log=TRUE)
    }

      if(prior.sens==1){
      prior.epsilon.i.cand <- dnorm(epsilon.i.cand, 0, 10, log=TRUE) 
      prior.epsilon.i <- dnorm(epsilon.i, 0, 10, log=TRUE)
      }
    
      if(prior.sens==2){
        prior.epsilon.i.cand <- dnorm(epsilon.i.cand, 0, 1.7, log=TRUE) 
        prior.epsilon.i <- dnorm(epsilon.i, 0, 1.7, log=TRUE)
      }
    
      if(prior.sens==3){
        prior.epsilon.i.cand <- dnorm(epsilon.i.cand, 0, 100, log=TRUE) 
        prior.epsilon.i <- dnorm(epsilon.i, 0, 100, log=TRUE)
      }
    
      ll.z.sum.cand <- sum(ll.z.cand)
      
      if(runif(1) < exp((ll.z.sum.cand + prior.epsilon.i.cand) -
                        (ll.z.sum + prior.epsilon.i))) {
        
        epsilon.i <- epsilon.i.cand
        epsilon <- epsilon.cand
        muz <- muz.cand
        ll.z<-ll.z.cand
        ll.z.sum <- ll.z.sum.cand
      }
    
      ## Metropolis update for epsilon (semi-perm sites)
      epsilon.s.cand <- rnorm(1, epsilon.s, tune[6])
      
      for(k in 2:nYears) {
        epsilon.cand[,k-1]<-plogis(epsilon.i*Inter + epsilon.s.cand*Semi + epsilon.p*Perm + w2*epsilon.c*pdsi[k-1]+ w3*eta[k-1])
        muz.cand[,k-1] <- z[,k-1]*(1-epsilon.cand[,k-1]*(1-gamma[,k-1])) + (1-z[,k-1])*gamma[,k-1] 
        muz.cand[,k-1] <- muz.cand[,k-1]*notFailed[,k]
        ll.z.cand[,k-1] <- dbinom(z[,k], 1, muz.cand[,k-1], log=TRUE)
      }
      if(prior.sens==1){
        prior.epsilon.s.cand <- dnorm(epsilon.s.cand, 0, 10, log=TRUE) 
        prior.epsilon.s <- dnorm(epsilon.s, 0, 10, log=TRUE)
      }
      
      if(prior.sens==2){
        prior.epsilon.s.cand <- dnorm(epsilon.s.cand, 0, 1.7, log=TRUE) 
        prior.epsilon.s <- dnorm(epsilon.s, 0, 1.7, log=TRUE)
      }
      
      if(prior.sens==3){
        prior.epsilon.s.cand <- dnorm(epsilon.s.cand, 0, 100, log=TRUE) 
        prior.epsilon.s <- dnorm(epsilon.s, 0, 100, log=TRUE)
      }
      
      ll.z.sum.cand <- sum(ll.z.cand)
      
      if(runif(1) < exp((ll.z.sum.cand + prior.epsilon.s.cand) -
                        (ll.z.sum + prior.epsilon.s))) {
        
        epsilon.s <- epsilon.s.cand
        epsilon <- epsilon.cand
        muz <- muz.cand
        ll.z<-ll.z.cand
        ll.z.sum <- ll.z.sum.cand
      }
    
      ## Metropolis update for epsilon (perm sites)
      epsilon.p.cand <- rnorm(1, epsilon.p, tune[7])
      
      for(k in 2:nYears) {
        epsilon.cand[,k-1]<-plogis(epsilon.i*Inter + epsilon.s*Semi + epsilon.p.cand*Perm + w2*epsilon.c*pdsi[k-1] + w3*eta[k-1])
        muz.cand[,k-1] <- z[,k-1]*(1-epsilon.cand[,k-1]*(1-gamma[,k-1])) + (1-z[,k-1])*gamma[,k-1] 
        muz.cand[,k-1] <- muz.cand[,k-1]*notFailed[,k]
        ll.z.cand[,k-1] <- dbinom(z[,k], 1, muz.cand[,k-1], log=TRUE)
      }
      if(prior.sens==1){
        prior.epsilon.p.cand <- dnorm(epsilon.p.cand, 0, 10, log=TRUE) 
        prior.epsilon.p <- dnorm(epsilon.p, 0, 10, log=TRUE)
      }
      
      if(prior.sens==2){
        prior.epsilon.p.cand <- dnorm(epsilon.p.cand, 0, 1.7, log=TRUE) 
        prior.epsilon.p <- dnorm(epsilon.p, 0, 1.7, log=TRUE)
      }
      
      if(prior.sens==3){
        prior.epsilon.p.cand <- dnorm(epsilon.p.cand, 0, 100, log=TRUE) 
        prior.epsilon.p <- dnorm(epsilon.p, 0, 100, log=TRUE)
      }
      
      ll.z.sum.cand <- sum(ll.z.cand)
      
      if(runif(1) < exp((ll.z.sum.cand + prior.epsilon.p.cand) -
                        (ll.z.sum + prior.epsilon.p))) {
        
        epsilon.p <- epsilon.p.cand
        epsilon <- epsilon.cand
        muz <- muz.cand
        ll.z<-ll.z.cand
        ll.z.sum <- ll.z.sum.cand
      }
    
      ## Metropolis update for epsilon (drought)
      epsilon.c.cand <- rnorm(1, epsilon.c, tune[8])
      
      for(k in 2:nYears) {
        epsilon.cand[,k-1]<-plogis(epsilon.i*Inter + epsilon.s*Semi + epsilon.p*Perm + w2*epsilon.c.cand*pdsi[k-1] + w3*eta[k-1])
        muz.cand[,k-1] <- z[,k-1]*(1-epsilon.cand[,k-1]*(1-gamma[,k-1])) + (1-z[,k-1])*gamma[,k-1] 
        muz.cand[,k-1] <- muz.cand[,k-1]*notFailed[,k]
        ll.z.cand[,k-1] <- dbinom(z[,k], 1, muz.cand[,k-1], log=TRUE)
      }

      if(w2==1){
        if(prior.sens==1){
          prior.epsilon.c.cand <- dnorm(epsilon.c.cand, 0, 10, log=TRUE) 
          prior.epsilon.c <- dnorm(epsilon.c, 0, 10, log=TRUE)
        }
        
        if(prior.sens==2){
          prior.epsilon.c.cand <- dnorm(epsilon.c.cand, 0, 1.7, log=TRUE) 
          prior.epsilon.c <- dnorm(epsilon.c, 0, 1.7, log=TRUE)
        }
        
        if(prior.sens==3){
          prior.epsilon.c.cand <- dnorm(epsilon.c.cand, 0, 100, log=TRUE) 
          prior.epsilon.c <- dnorm(epsilon.c, 0, 100, log=TRUE)
        }
      }
      if(w2==0){
        prior.epsilon.c.cand <- dnorm(epsilon.c.cand, 0.62, 0.63, log=TRUE)
        prior.epsilon.c <- dnorm(epsilon.c, 0.62, 0.63, log=TRUE)
      }
      ll.z.sum.cand <- sum(ll.z.cand)
      
      if(runif(1) < exp((ll.z.sum.cand + prior.epsilon.c.cand) -
                        (ll.z.sum + prior.epsilon.p))) {
        
        epsilon.c <- epsilon.c.cand
        epsilon <- epsilon.cand
        muz <- muz.cand
        ll.z<-ll.z.cand
        ll.z.sum <- ll.z.sum.cand
      }
      
      ## Metropolis update for epsilon (environmental stochasticity)
      sdEta.cand <- rnorm(1, sdEta, tune[16])
      
      if(sdEta.cand > 0) {
        
        ll.sdEta.cand<-sum(dnorm(eta, 0, sdEta.cand, log=TRUE), na.rm=TRUE)
        ll.sdEta<-sum(dnorm(eta, 0, sdEta, log=TRUE), na.rm=TRUE)
        
        ## Priors
        prior.sdEta.cand <- dgamma(sdEta.cand, 1, 1, log=TRUE)
        prior.sdEta <- dgamma(sdEta.cand, 1, 1, log=TRUE)
        
        if(runif(1) < exp((ll.sdEta.cand + prior.sdEta.cand) -
                          (ll.sdEta + prior.sdEta))) {
          sdEta <- sdEta.cand
        }
      }
      
      ##Metropolis update for eta (random effect of "climate")
      ll.etaY<-ll.etaY.cand<-matrix(0, nSites, nYears-1)
      for(k in 2:nYears) {
        eta.cand[k-1]<-rnorm(1, eta[k-1], tune[17])
        epsilon.cand[,k-1]<-plogis(epsilon.i*Inter + epsilon.s*Semi + epsilon.p*Perm + w2*epsilon.c*pdsi[k-1] + w3*eta.cand[k-1])
        muz.cand[,k-1] <- z[,k-1]*(1-epsilon.cand[,k-1]*(1-gamma[,k-1])) + (1-z[,k-1])*gamma[,k-1]
        muz.cand[,k-1] <- muz.cand[,k-1]*notFailed[,k]
        ll.z.cand <- ll.z 
        ll.z.cand[,k-1] <- dbinom(z[,k], 1, muz.cand[,k-1], log=TRUE)
        
        ll.z.sum.cand <- sum(ll.z.cand)
        
        if(w3==1){
        prior.eta.cand <- sum(dnorm(eta.cand, 0, sdEta, log=TRUE))
        prior.eta <- sum(dnorm(eta[k-1], 0, sdEta, log=TRUE))
        }
      
        if(w3==0){
          prior.eta.cand <- sum(dnorm(eta.cand, 0.002, 0.26, log=TRUE))
          prior.eta <- sum(dnorm(eta[k-1], 0.002, 0.26, log=TRUE))
        }
      
        if(runif(1) < exp((ll.z.sum.cand + prior.eta.cand) -
                          (ll.z.sum + prior.eta))) {
          
          eta[k-1] <- eta.cand[k-1]
          epsilon[,k-1] <- epsilon.cand[,k-1]
          muz[,k-1] <- muz.cand[,k-1]
          ll.z<-ll.z.cand
          ll.z.sum <- ll.z.sum.cand
        }
      }
    
    ## Update for occupancy state at each site, each year
    zkup <- rep(0, nYears-1)
    for(k in 2:nYears) {
       anyDet <- anyDetections[,k]==1 
       zknown <- anyDet | !notFailed[,k] 
       prop.back <- prop.cand <- 0
       for(i in 1:nSites) {
           if(zknown[i])
               next
           ## Reject highly unlikely proposals (before proposing them)
           ## This speed trick shouldn't affect anything but
           ## can double check by changing tolerance (tol)
           if(z[i,k]<1 & muz[i,k-1]<tol)
               next
           zk.wide <- matrix(z[,k], nSites, nReps)
           zk.cand <- z[,k]
           zk.cand[i] <- 1-z[i,k]
           zk.cand.wide <- matrix(zk.cand, nSites, nReps)

           ll.y.tmp <- 0
           ll.y.cand.tmp <- 0
           if((k > 4) & (i <= nSampled)) { 
               ll.y.cand.tmp <- dbinom(y[i,,k], 1, zk.cand[i]*p[i,,k], log=TRUE)
               ll.y.tmp <- sum(ll.y[i,,k], na.rm=TRUE)
           }
           ll.z.cand[i,k-1] <- dbinom(zk.cand[i], 1, muz[i,k-1], log=TRUE)
           ll.z2 <- ll.z2.cand <- 0
           if(k < nYears) {
               zkt.cand <- matrix(zk.cand, nSites, nSites, byrow=TRUE)
               gamma.cand[,k] <- 1 - exp(rowSums(log(1-G*zkt.cand)))
               muz.cand[,k] <- (zk.cand*(1-epsilon[,k]*(1-gamma.cand[,k])) + (1-zk.cand)*gamma.cand[,k])*notFailed[,k+1]
               ll.z.cand[,k] <- dbinom(z[,k+1], 1, muz.cand[,k], log=TRUE)
               ll.z2 <- sum(ll.z[,k])
               ll.z2.cand <- sum(ll.z.cand[,k])
           }
           if(runif(1) < exp((sum(ll.y.cand.tmp, na.rm=TRUE) + ll.z.cand[i,k-1] +
                              ll.z2.cand + prop.back) -
                             (ll.y.tmp + ll.z[i,k-1] +
                              ll.z2 + prop.cand))) {
               z[,k] <- zk.cand
               ll.z[i,k-1] <- ll.z.cand[i,k-1]
               if(k < nYears) {
                   epsilon[,k] <- epsilon.cand[,k]
                   gamma[,k] <- gamma.cand[,k]
                   muz[,k] <- muz.cand[,k]
                   ll.z[,k] <- ll.z.cand[,k]
               }
               if((i <= nSampled) & (k>4)) {
                   ll.y[i,,k] <- ll.y.cand.tmp
               }
               zkup[k-1] <- zkup[k-1] + 1
           }
       }
   }

    nz1 <- nz1+z



    #Update for beta0
    beta0.cand<-rnorm(1, beta0, tune[9])
    p.cand <- plogis(beta0.cand + beta1*p.cov1 + beta2*p.cov2)
    z.wide <- z[,rep(1:nYears, each=nReps)]
    z.a <- array(z.wide, c(nSites, nReps, nYears))

    ll.y[,,dataYears] <- dbinom(y[,,dataYears], 1, z.a[1:nSampled,,dataYears]*p[,,dataYears], log=TRUE)
    ll.y.cand[,,dataYears] <- dbinom(y[,,dataYears], 1, z.a[1:nSampled,,dataYears]*p.cand[,,dataYears], log=TRUE)
    if(prior.sens==1){
      prior.beta0.cand <- dnorm(beta0.cand, 0, 10, log=TRUE) 
      prior.beta0 <- dnorm(beta0, 0, 10, log=TRUE)
    }
    
    if(prior.sens==2){
      prior.beta0.cand <- dnorm(beta0.cand, 0, 1.7, log=TRUE) 
      prior.beta0 <- dnorm(beta0, 0, 1.7, log=TRUE)
    }
    
    if(prior.sens==3){
      prior.beta0.cand <- dnorm(beta0.cand, 0, 100, log=TRUE) 
      prior.beta0 <- dnorm(beta0, 0, 100, log=TRUE)
    }

    ll.y.sum <- sum(ll.y, na.rm=TRUE)
    ll.y.sum.cand <- sum(ll.y.cand, na.rm=TRUE)

    if(runif(1) < exp((ll.y.sum.cand + prior.beta0.cand) -
                      (ll.y.sum + prior.beta0))) {
        beta0 <- beta0.cand
        p <- p.cand
        ll.y <- ll.y.cand
        ll.y.sum <- ll.y.sum.cand
    }

    #Update for beta1
    beta1.cand<-rnorm(1, beta1, tune[10])
    p.cand <- plogis(beta0 + beta1.cand*p.cov1 + beta2*p.cov2)
    z.wide <- z[,rep(1:nYears, each=nReps)]
    z.a <- array(z.wide, c(nSites, nReps, nYears))

    ll.y.cand[,,dataYears] <- dbinom(y[,,dataYears], 1, z.a[1:nSampled,,dataYears]*p.cand[,,dataYears], log=TRUE)
    if(prior.sens==1){
      prior.beta1.cand <- dnorm(beta1.cand, 0, 10, log=TRUE) 
      prior.beta1 <- dnorm(beta1, 0, 10, log=TRUE)
    }
    
    if(prior.sens==2){
      prior.beta1.cand <- dnorm(beta1.cand, 0, 1.7, log=TRUE) 
      prior.beta1 <- dnorm(beta1, 0, 1.7, log=TRUE)
    }
    
    if(prior.sens==3){
      prior.beta1.cand <- dnorm(beta1.cand, 0, 100, log=TRUE) 
      prior.beta1 <- dnorm(beta1, 0, 100, log=TRUE)
    }

    ll.y.sum.cand <- sum(ll.y.cand, na.rm=TRUE)

    if(runif(1) < exp((ll.y.sum.cand + prior.beta1.cand) -
                      (ll.y.sum + prior.beta1))) {
        beta1 <- beta1.cand
        p <- p.cand
        ll.y <- ll.y.cand
        ll.y.sum <- ll.y.sum.cand
    }

    #Update for beta2
    beta2.cand<-rnorm(1, beta2, tune[11])
    p.cand <- plogis(beta0 + beta1*p.cov1 + beta2.cand*p.cov2)
    z.wide <- z[,rep(1:nYears, each=nReps)]
    z.a <- array(z.wide, c(nSites, nReps, nYears))

    ll.y.cand[,,dataYears] <- dbinom(y[,,dataYears], 1, z.a[1:nSampled,,dataYears]*p.cand[,,dataYears], log=TRUE)
    if(prior.sens==1){
      prior.beta2.cand <- dnorm(beta2.cand, 0, 10, log=TRUE) 
      prior.beta2 <- dnorm(beta2, 0, 10, log=TRUE)
    }
    
    if(prior.sens==2){
      prior.beta2.cand <- dnorm(beta2.cand, 0, 1.7, log=TRUE) 
      prior.beta2 <- dnorm(beta2, 0, 1.7, log=TRUE)
    }
    
    if(prior.sens==3){
      prior.beta2.cand <- dnorm(beta2.cand, 0, 100, log=TRUE) 
      prior.beta2 <- dnorm(beta2, 0, 100, log=TRUE)
    }
    
    ll.y.sum.cand <- sum(ll.y.cand, na.rm=TRUE)

    if(runif(1) < exp((ll.y.sum.cand + prior.beta2.cand) -
                      (ll.y.sum + prior.beta2))) {
        beta2 <- beta2.cand
        p <- p.cand
        ll.y <- ll.y.cand
        ll.y.sum <- ll.y.sum.cand
    }

    zk <- colSums(z)

    samples[s,] <- c(w1, w2, w3, sigma, gamma0.i, gamma0.s, gamma0.p,
                     epsilon.i, epsilon.s, epsilon.p, epsilon.c, sdEta,
                     beta0, beta1, beta2, alpha1, eta=eta, zk=zk, deviance=-2*ll.y.sum, deviance.latent=-2*ll.z.sum)

    zK[,s] <- z[,nYears]
    if(monitor.z)
        zA[,,s] <- z
    if(calc.waic)
      lya[,,,s]<-ll.y
    lza[,,s]<-ll.z
  }

  final.state <- list(z=z, D=D, G=G, eta=eta, samples=samples[s,])
  library(coda)
  return(list(samples=samples, final.state=final.state,
              zK=zK, zA=zA, Ez=nz1/nIter,
              lya=lya,
              lza=lza,
              seed=.Random.seed))
}

dynroccHC <- cmpfun(dynroccH)

#This software has been approved for release by the U.S. Geological Survey (USGS). Although the software has been subjected to rigorous review, the software has not been approved as a software release, and is only intended to document methodology.  The USGS reserves the right to update the software as needed pursuant to further analysis and review. No warranty, expressed or implied, is made by the USGS or the U.S. Government as to the functionality of the software and related material nor shall the fact of release constitute any such warranty. Furthermore, the software is released on condition that neither the USGS nor the U.S. Government shall be held liable for any damages resulting from its authorized or unauthorized use.
